@@ -24,6 +24,9 @@
 #include "MountPoints.h"
 #include "Exception.h"
 
+#define DONT_TRUST_NTFS_HARD_LINKS      1
+#define VERBOSE_NTFS_HARD_LINKS         0
+
 using namespace QDirStat;
 
 
@@ -42,8 +45,19 @@ DirReadJob::DirReadJob( DirTree * tree,
 
 DirReadJob::~DirReadJob()
 {
-    if ( _dir )
-	_dir->readJobFinished( _dir );
+    if ( ! _tree->beingDestroyed() )
+    {
+	// Only do this if the tree is not in the process of being destroyed;
+	// otherwise all FileInfo / DirInfo pointers pointing into that tree
+	// may already be invalid. And even if they are not, it is now
+	// pointless to do all the housekeeping stuff to finalize the read job:
+	// We'd be beautifying the tree content that is now being destroyed.
+	//
+	// https://github.com/shundhammer/qdirstat/issues/122
+
+	if ( _dir )
+	    _dir->readJobFinished( _dir );
+    }
 }
 
 
@@ -92,7 +106,7 @@ void DirReadJob::deletingChild( FileInfo *deletedChild )
 }
 
 
-bool DirReadJob::crossingFileSystems( DirInfo * parent, DirInfo * child )
+bool DirReadJob::crossingFilesystems( DirInfo * parent, DirInfo * child )
 {
     if ( parent->device() == child->device() )
 	return false;
@@ -102,15 +116,15 @@ bool DirReadJob::crossingFileSystems( DirInfo * parent, DirInfo * child )
 
     if ( childDevice.isEmpty() && parent->readState() == DirCached )
     {
-        // A DirInfo from a cache file always has device number 0, so the
-        // initial check failed because of that. The child might still be a
-        // mount point, but since that path was not found in the list of known
-        // mount points from /proc/mounts or /etc/mtab, we assume that this was
-        // not the case.
-        //
-        // See also https://github.com/shundhammer/qdirstat/issues/114
+	// A DirInfo from a cache file always has device number 0, so the
+	// initial check failed because of that. The child might still be a
+	// mount point, but since that path was not found in the list of known
+	// mount points from /proc/mounts or /etc/mtab, we assume that this was
+	// not the case.
+	//
+	// See also https://github.com/shundhammer/qdirstat/issues/114
 
-        return false;
+	return false;
     }
 
     if ( parentDevice.isEmpty() )
@@ -123,7 +137,7 @@ bool DirReadJob::crossingFileSystems( DirInfo * parent, DirInfo * child )
 
     if ( crossing )
     {
-	logInfo() << "File system boundary at mount point " << child
+	logInfo() << "Filesystem boundary at mount point " << child
 		  << " on device " << ( childDevice.isEmpty() ? "<unknown>" : childDevice )
 		  << endl;
     }
@@ -143,7 +157,7 @@ QString DirReadJob::device( const DirInfo * dir ) const
 
     if ( dir )
     {
-	const MountPoint * mountPoint = MountPoints::findByPath( dir->url() );
+	MountPoint * mountPoint = MountPoints::findByPath( dir->url() );
 
 	if ( mountPoint )
 	    device = mountPoint->device();
@@ -153,13 +167,34 @@ QString DirReadJob::device( const DirInfo * dir ) const
 }
 
 
+bool DirReadJob::shouldCrossIntoFilesystem( const DirInfo * dir ) const
+{
+    MountPoint * mountPoint = MountPoints::findByPath( dir->url() );
 
+    bool doCross =
+	! mountPoint->isSystemMount()  &&	//  /dev, /proc, /sys, ...
+	! mountPoint->isDuplicate()    &&	//  bind mount or multiple mounted
+	! mountPoint->isNetworkMount();		//  NFS or CIFS (Samba)
+
+    logDebug() << ( doCross ? "Reading" : "Not reading" )
+	       << " mounted filesystem " << mountPoint->path() << endl;
+
+    return doCross;
+}
+
+
+
+
+
+bool LocalDirReadJob::_warnedAboutNtfsHardLinks = false;
 
 
 LocalDirReadJob::LocalDirReadJob( DirTree * tree,
-				  DirInfo * dir )
-    : DirReadJob( tree, dir )
-    , _applyFileChildExcludeRules( false )
+				  DirInfo * dir ):
+    DirReadJob( tree, dir ),
+    _applyFileChildExcludeRules( false ),
+    _checkedForNtfs( false ),
+    _isNtfs( false )
 {
     if ( _dir )
 	_dirName = _dir->url();
@@ -168,6 +203,7 @@ LocalDirReadJob::LocalDirReadJob( DirTree * tree,
 
 LocalDirReadJob::~LocalDirReadJob()
 {
+    // NOP
 }
 
 
@@ -263,6 +299,28 @@ void LocalDirReadJob::startReading()
 			    return;
 		    }
 
+#if DONT_TRUST_NTFS_HARD_LINKS
+
+                    if ( statInfo.st_nlink > 1 && isNtfs() )
+                    {
+                        // NTFS seems to return bogus hard link counts; use 1 instead.
+                        // See  https://github.com/shundhammer/qdirstat/issues/88
+
+#if ! VERBOSE_NTFS_HARD_LINKS
+                        if ( ! _warnedAboutNtfsHardLinks )
+#endif
+                        {
+                            logWarning() << "Not trusting NTFS with hard links: \""
+                                         << _dir->url() << "/" << entryName
+                                         << "\" links: " << statInfo.st_nlink
+                                         << " -> resetting to 1"
+                                         << endl;
+                            _warnedAboutNtfsHardLinks = true;
+                        }
+
+                        statInfo.st_nlink = 1;
+                    }
+#endif
 		    FileInfo * child = new FileInfo( entryName, &statInfo, _tree, _dir );
 		    CHECK_NEW( child );
 
@@ -342,7 +400,7 @@ void LocalDirReadJob::processSubDir( const QString & entryName, DirInfo * subDir
     }
     else // No exclude rule matched
     {
-	if ( ! crossingFileSystems(_dir, subDir ) ) // normal case
+	if ( ! crossingFilesystems(_dir, subDir ) ) // normal case
 	{
 	    LocalDirReadJob * job = new LocalDirReadJob( _tree, subDir );
 	    CHECK_NEW( job );
@@ -353,7 +411,7 @@ void LocalDirReadJob::processSubDir( const QString & entryName, DirInfo * subDir
 	{
 	    subDir->setMountPoint();
 
-	    if ( _tree->crossFileSystems() )
+	    if ( _tree->crossFilesystems() && shouldCrossIntoFilesystem( subDir ) )
 	    {
 		LocalDirReadJob * job = new LocalDirReadJob( _tree, subDir );
 		CHECK_NEW( job );
@@ -412,12 +470,12 @@ bool LocalDirReadJob::readCacheFile( const QString & cacheFileName )
 
 	    _tree->clearAndReadCache( cacheFullName );
 
-            // Since this clears the tree and thus the job queue and thus
-            // deletes this read job, it is important not to do anything after
-            // this point that might access any member variables or even just
-            // uses any virtual method.
+	    // Since this clears the tree and thus the job queue and thus
+	    // deletes this read job, it is important not to do anything after
+	    // this point that might access any member variables or even just
+	    // uses any virtual method.
 
-            return true;
+	    return true;
 	}
 	else
 	{
@@ -554,6 +612,24 @@ FileInfo * LocalDirReadJob::stat( const QString & url,
 }
 
 
+bool LocalDirReadJob::isNtfs()
+{
+    if ( ! _checkedForNtfs )
+    {
+        _isNtfs         = false;
+        _checkedForNtfs = true;
+
+        if ( ! _dirName.isEmpty() )
+        {
+            MountPoint * mountPoint = MountPoints::findNearestMountPoint( _dirName );
+            _isNtfs = mountPoint && mountPoint->isNtfs();
+        }
+    }
+
+    return _isNtfs;
+}
+
+
 
 
 
