@@ -25,6 +25,7 @@ namespace QDirStat
     class DirTree;
     class CacheReader;
     class DirReadJobQueue;
+    class MountPoint;
 
 
     /**
@@ -137,10 +138,10 @@ namespace QDirStat
 	void finished();
 
 	/**
-	 * Check if going from 'parent' to 'child' would cross a file system
+	 * Check if going from 'parent' to 'child' would cross a filesystem
 	 * boundary. This take Btrfs subvolumes into account.
 	 **/
-	bool crossingFileSystems( DirInfo * parent, DirInfo * child );
+	bool crossingFilesystems( DirInfo * parent, DirInfo * child );
 
 	/**
 	 * Return the device name where 'dir' is on if it's a mount point.
@@ -148,6 +149,13 @@ namespace QDirStat
 	 **/
 	QString device( const DirInfo * dir ) const;
 
+	/**
+	 * Check if we really should cross into a mounted filesystem; don't do
+	 * it if this is a system mount, a bind mount, a filesystem mounted
+	 * multiple times, or a network mount (NFS / Samba).
+	 **/
+	bool shouldCrossIntoFilesystem( const DirInfo * dir ) const;
+
 
 	DirTree *	   _tree;
 	DirInfo *	   _dir;
@@ -187,9 +195,9 @@ namespace QDirStat
      *
      * This will use lstat() system calls rather than KDE's network transparent
      * directory services since lstat() unlike the KDE services can obtain
-     * information about the device (i.e. file system) a file or directory
+     * information about the device (i.e. filesystem) a file or directory
      * resides on. This is important if you wish to limit directory scans to
-     * one file system - which is most desirable when that one file system runs
+     * one filesystem - which is most desirable when that one filesystem runs
      * out of space.
      *
      * @short Directory reader that reads one local directory.
@@ -307,6 +315,11 @@ namespace QDirStat
 	 **/
 	QString fullName( const QString & entryName ) const;
 
+	/**
+	 * Return 'true' if the current filesystem is NTFS.
+	 **/
+	bool isNtfs();
+
 
 	//
 	// Data members
@@ -314,6 +327,10 @@ namespace QDirStat
 
 	QString _dirName;
 	bool	_applyFileChildExcludeRules;
+	bool	_checkedForNtfs;
+	bool	_isNtfs;
+
+	static bool _warnedAboutNtfsHardLinks;
 
     };	// LocalDirReadJob
 
