@@ -36,7 +36,7 @@ namespace QDirStat
      * This class provides some infrastructure as well as global data for a
      * directory tree. It acts as the glue that holds things together: The root
      * item from which to descend into the subtrees, the read queue and some
-     * global policies (like whether or not to cross file systems while reading
+     * global policies (like whether or not to cross filesystems while reading
      * directories).
      *
      * Notice that this class uses a "pseudo root" to better conform with Qt's
@@ -194,19 +194,19 @@ namespace QDirStat
 	void unblock( DirReadJob * job );
 
 	/**
-	 * Should directory scans cross file systems?
+	 * Should directory scans cross filesystems?
 	 *
 	 * Notice: This can only be avoided with local directories where the
 	 * device number a file resides on can be obtained.
 	 * Remember, that's what this QDirStat business is all about.  ;-)
 	 **/
-	bool crossFileSystems() const { return _crossFileSystems; }
+	bool crossFilesystems() const { return _crossFilesystems; }
 
 	/**
-	 * Set or unset the "cross file systems" flag.
+	 * Set or unset the "cross filesystems" flag.
 	 **/
-	void setCrossFileSystems( bool doCross )
-	    { _crossFileSystems = doCross; }
+	void setCrossFilesystems( bool doCross )
+	    { _crossFilesystems = doCross; }
 
 	/**
 	 * Notification that a child has been added.
@@ -276,9 +276,9 @@ namespace QDirStat
 	 **/
 	void readCache( const QString & cacheFileName );
 
-        /**
-         * Clear the tree and read a cache file.
-         **/
+	/**
+	 * Clear the tree and read a cache file.
+	 **/
 	void clearAndReadCache( const QString & cacheFileName );
 
 	/**
@@ -326,7 +326,7 @@ namespace QDirStat
 
 	/**
 	 * Iterate over all filters and return 'true' if any of them wants a
-	 * file system object to be ignored during directory reading, 'false'
+	 * filesystem object to be ignored during directory reading, 'false'
 	 * if not.
 	 **/
 	bool checkIgnoreFilters( const QString & path );
@@ -336,6 +336,30 @@ namespace QDirStat
 	 **/
 	bool hasFilters() const { return ! _filters.isEmpty(); }
 
+	/**
+	 * Return 'true' if this DirTree is in the process of being destroyed,
+	 * so any FileInfo / DirInfo pointers stored outside the tree might
+	 * have become invalid.
+	 **/
+	bool beingDestroyed() const { return _beingDestroyed; }
+
+        /**
+         * Return the number of 512-bytes blocks per cluster.
+         *
+         * This may be 0 if no small file (< 512 bytes) was found in this tree
+         * yet.
+         **/
+        int blocksPerCluster() const { return _blocksPerCluster; }
+
+        /**
+         * Return the cluster size of this tree, i.e. the disk space allocation
+         * unit. No non-zero file can have an allocated size smaller than this.
+         *
+         * This may be 0 if no small file (< 512 bytes) was found in this tree
+         * yet.
+         **/
+        FileSize clusterSize() const { return _blocksPerCluster * STD_BLOCK_SIZE; }
+
 
     signals:
 
@@ -444,18 +468,26 @@ namespace QDirStat
 	 **/
 	void recalc( DirInfo * dir );
 
+        /**
+         * Try to derive the cluster size from 'item'.
+         **/
+        void detectClusterSize( FileInfo * item );
+
 
 
 	// Data members
 
 	DirInfo *		_root;
 	DirReadJobQueue		_jobQueue;
-	bool			_crossFileSystems;
+	bool			_crossFilesystems;
 	bool			_isBusy;
 	QString			_device;
 	QString			_url;
 	ExcludeRules *		_excludeRules;
 	QList<DirTreeFilter *>	_filters;
+	bool			_beingDestroyed;
+        bool                    _haveClusterSize;
+        int                     _blocksPerCluster;
 
     };	// class DirTree
 
