@@ -8,9 +8,9 @@
 
 
 #include <QPalette>
-#include <QGuiApplication>
 
 #include "Qt4Compat.h"
+
 #include "DirTreeModel.h"
 #include "DirTree.h"
 #include "FileInfoIterator.h"
@@ -22,6 +22,14 @@
 #include "Exception.h"
 #include "DebugHelpers.h"
 
+// Number of clusters up to which a file will be considered small and will also
+// display the allocated size like (4k).
+#define SMALL_FILE_CLUSTERS     2
+
+// Used used percent below which a small file will also display the allocated size
+// like (4k)
+#define SMALL_FILE_SHOW_ALLOC_THRESHOLD         75
+
 using namespace QDirStat;
 
 
@@ -61,7 +69,8 @@ void DirTreeModel::readSettings()
     Settings settings;
     settings.beginGroup( "DirectoryTree" );
 
-    _tree->setCrossFileSystems( settings.value( "CrossFileSystems", false ).toBool() );
+    _tree->setCrossFilesystems	( settings.value( "CrossFilesystems", false ).toBool() );
+    FileInfo::setIgnoreHardLinks( settings.value( "IgnoreHardLinks",  false ).toBool() );
     _treeIconDir	 = settings.value( "TreeIconDir" , ":/icons/tree-medium/" ).toString();
     _updateTimerMillisec = settings.value( "UpdateTimerMillisec", 333 ).toInt();
     _slowUpdateMillisec	 = settings.value( "SlowUpdateMillisec", 3000 ).toInt();
@@ -96,9 +105,10 @@ void DirTreeModel::writeSettings()
 
     settings.setValue( "SlowUpdateMillisec", _slowUpdateMillisec  );
 
-    settings.setDefaultValue( "CrossFileSystems",    _tree ? _tree->crossFileSystems() : false );
-    settings.setDefaultValue( "TreeIconDir",	     _treeIconDir	  );
-    settings.setDefaultValue( "UpdateTimerMillisec", _updateTimerMillisec );
+    settings.setDefaultValue( "CrossFilesystems",    _tree ? _tree->crossFilesystems() : false );
+    settings.setDefaultValue( "IgnoreHardLinks",     FileInfo::ignoreHardLinks() );
+    settings.setDefaultValue( "TreeIconDir",	     _treeIconDir		 );
+    settings.setDefaultValue( "UpdateTimerMillisec", _updateTimerMillisec	 );
 
     settings.endGroup();
 
@@ -215,18 +225,19 @@ void DirTreeModel::loadIcons()
     if ( ! _treeIconDir.endsWith( "/" ) )
 	_treeIconDir += "/";
 
-    _dirIcon	       = QPixmap( _treeIconDir + "dir.png"	      );
-    _dotEntryIcon      = QPixmap( _treeIconDir + "dot-entry.png"      );
-    _fileIcon	       = QPixmap( _treeIconDir + "file.png"	      );
-    _symlinkIcon       = QPixmap( _treeIconDir + "symlink.png"	      );
-    _unreadableDirIcon = QPixmap( _treeIconDir + "unreadable-dir.png" );
-    _mountPointIcon    = QPixmap( _treeIconDir + "mount-point.png"    );
-    _stopIcon	       = QPixmap( _treeIconDir + "stop.png"	      );
-    _excludedIcon      = QPixmap( _treeIconDir + "excluded.png"	      );
-    _blockDeviceIcon   = QPixmap( _treeIconDir + "block-device.png"   );
-    _charDeviceIcon    = QPixmap( _treeIconDir + "char-device.png"    );
-    _specialIcon       = QPixmap( _treeIconDir + "special.png"	      );
-    _pkgIcon	       = QPixmap( _treeIconDir + "folder-pkg.png"     );
+    _dirIcon	       = QIcon( _treeIconDir + "dir.png"	    );
+    _dotEntryIcon      = QIcon( _treeIconDir + "dot-entry.png"      );
+    _fileIcon	       = QIcon( _treeIconDir + "file.png"	    );
+    _symlinkIcon       = QIcon( _treeIconDir + "symlink.png"	    );
+    _unreadableDirIcon = QIcon( _treeIconDir + "unreadable-dir.png" );
+    _mountPointIcon    = QIcon( _treeIconDir + "mount-point.png"    );
+    _stopIcon	       = QIcon( _treeIconDir + "stop.png"	    );
+    _excludedIcon      = QIcon( _treeIconDir + "excluded.png"	    );
+    _blockDeviceIcon   = QIcon( _treeIconDir + "block-device.png"   );
+    _charDeviceIcon    = QIcon( _treeIconDir + "char-device.png"    );
+    _specialIcon       = QIcon( _treeIconDir + "special.png"	    );
+    _pkgIcon	       = QIcon( _treeIconDir + "folder-pkg.png"     );
+    _atticIcon         = _dirIcon;
 }
 
 
@@ -288,6 +299,20 @@ int DirTreeModel::rowNumber( FileInfo * child ) const
 }
 
 
+FileInfo * DirTreeModel::itemFromIndex( const QModelIndex & index )
+{
+    FileInfo * item = 0;
+
+    if ( index.isValid() )
+    {
+        item = static_cast<FileInfo *>( index.internalPointer() );
+        CHECK_MAGIC( item );
+    }
+
+    return item;
+}
+
+
 
 //
 // Reimplented from QAbstractItemModel
@@ -343,7 +368,7 @@ int DirTreeModel::rowCount( const QModelIndex & parentIndex ) const
 	    break;
 
 	case DirError:
-        case DirPermissionDenied:
+	case DirPermissionDenied:
 
 	    // This is a hybrid case: Depending on the dir reader, the dir may
 	    // or may not be finished at this time. For a local dir, it most
@@ -474,10 +499,10 @@ QVariant DirTreeModel::data( const QModelIndex & index, int role ) const
 			    }
 			    else
 			    {
-				return item->subtreePercent();
+				return item->subtreeAllocatedPercent();
 			    }
 			}
-		    case PercentNumCol:	      return item->subtreePercent();
+		    case PercentNumCol:	      return item->subtreeAllocatedPercent();
 		    case SizeCol:	      return item->totalSize();
 		    case TotalItemsCol:	      return item->totalItems();
 		    case TotalFilesCol:	      return item->totalFiles();
@@ -709,18 +734,18 @@ QVariant DirTreeModel::columnText( FileInfo * item, int col ) const
 	return QVariant();
 
     if ( item->isPkgInfo() &&
-         item->readState() == DirAborted &&
-         ! item->firstChild() &&
-         col != NameCol )
+	 item->readState() == DirAborted &&
+	 ! item->firstChild() &&
+	 col != NameCol )
     {
-        return "?";
+	return "?";
     }
 
     switch ( col )
     {
 	case NameCol:		  return item->name();
 	case PercentBarCol:	  return item->isExcluded() ? tr( "[Excluded]" ) : QVariant();
-	case PercentNumCol:	  return item == _tree->firstToplevel() ? QVariant() : formatPercent( item->subtreePercent() );
+	case PercentNumCol:	  return item == _tree->firstToplevel() ? QVariant() : formatPercent( item->subtreeAllocatedPercent() );
 	case SizeCol:		  return sizeColText( item );
 	case LatestMTimeCol:	  return QString( "  " ) + formatTime( item->latestMtime() );
 	case UserCol:		  return limitedInfo ? QVariant() : item->userName();
@@ -733,16 +758,16 @@ QVariant DirTreeModel::columnText( FileInfo * item, int col ) const
     {
 	if ( item->readError() )
 	{
-            switch ( col )
-            {
-                case TotalItemsCol:
-                case TotalFilesCol:
-                case TotalSubDirsCol:
-                    return "?";
+	    switch ( col )
+	    {
+		case TotalItemsCol:
+		case TotalFilesCol:
+		case TotalSubDirsCol:
+		    return "?";
 
-                default:
-                    break;
-            }
+		default:
+		    break;
+	    }
 	}
 
 	QString prefix = item->sizePrefix();
@@ -757,7 +782,7 @@ QVariant DirTreeModel::columnText( FileInfo * item, int col ) const
 		else
 		    return prefix + QString( "%1" ).arg( item->totalSubDirs() );
 
-	    case OldestFileMTimeCol:  return QString( "  " ) + formatTime( item->oldestFileMtime() );
+	    case OldestFileMTimeCol:  return QString( "	 " ) + formatTime( item->oldestFileMtime() );
 	}
     }
 
@@ -782,52 +807,124 @@ int DirTreeModel::directChildrenCount( FileInfo * subtree ) const
 QString DirTreeModel::sizeText( FileInfo * item, QString (*fmtSz)(FileSize) )
 {
     if ( ! item->isFile() )
-        return "";
+	return "";
 
     QString text;
 
     if ( item->links() > 1 )  // Multiple hard links
     {
-        if ( item->isSparseFile() )
-        {
-            text = tr( "%1 / %2 Links (allocated: %3)" )
-                .arg( fmtSz( item->byteSize() ) )
-                .arg( item->links() )
-                .arg( fmtSz( item->allocatedSize() ) );
-        }
-        else
-        {
-            text = tr( "%1 / %2 Links" )
-                .arg( fmtSz( item->byteSize() ) )
-                .arg( item->links() );
-        }
+	if ( item->isSparseFile() )
+	{
+	    text = tr( "%1 / %2 Links (allocated: %3)" )
+		.arg( fmtSz( item->rawByteSize() ) )
+		.arg( item->links() )
+		.arg( fmtSz( item->rawAllocatedSize() ) );
+	}
+	else
+	{
+	    text = tr( "%1 / %2 Links" )
+		.arg( fmtSz( item->rawByteSize() ) )
+		.arg( item->links() );
+	}
     }
     else // No multiple hard links
     {
-        if ( item->isSparseFile() )
-        {
-            text = tr( "%1 (allocated: %2)" )
-                .arg( fmtSz( item->byteSize() ) )
-                .arg( fmtSz( item->allocatedSize() ) );
-        }
+	if ( item->isSparseFile() )
+	{
+	    text = tr( "%1 (allocated: %2)" )
+		.arg( fmtSz( item->rawByteSize() ) )
+		.arg( fmtSz( item->rawAllocatedSize() ) );
+	}
     }
 
     return text;
 }
 
 
+QString DirTreeModel::smallSizeText( FileInfo * item )
+{
+    if ( ! item->isFile() )
+	return "";
+
+    FileSize allocated = item->allocatedSize();
+    FileSize size      = item->size();
+    QString  text;
+
+    if ( allocated >= 1024 )                    // at least 1k so the (?k) makes sense
+    {
+        if ( allocated % 1024 == 0   &&         // if it's really even kB
+             allocated < 1024 * 1024   )        // and below 1 MB
+             // && item->usedPercent() < SMALL_FILE_SHOW_ALLOC_THRESHOLD &&
+        {
+            if ( size < 1024 )
+            {
+                text = QString( "%1 B (%2k)" )
+                    .arg( size )
+                    .arg( allocated / 1024 );
+            }
+            else
+            {
+                text = QString( "%1 (%2k)" )
+                    .arg( formatSize( size ) )
+                    .arg( allocated / 1024 );
+            }
+        }
+    }
+
+    if ( text.isEmpty() )
+        return formatSize( size );
+
+    return text;
+}
+
+
+bool DirTreeModel::isSmallFile( FileInfo * item )
+{
+    if ( item			&&
+         item->isFile()		&&
+         item->blocks() > 0     &&
+         ! item->isSparseFile() &&
+         item->tree()		  )
+    {
+        FileSize clusterSize = item->tree()->clusterSize();
+
+	if ( clusterSize > 0 )
+        {
+            if ( item->allocatedSize() <= clusterSize * SMALL_FILE_CLUSTERS )
+                return true;
+
+            if ( item->allocatedSize() <= clusterSize * ( SMALL_FILE_CLUSTERS + 1 ) )
+            {
+                FileSize unused = item->allocatedSize() - item->rawByteSize();
+
+                // 'unused' might be negative for sparse files, but the check
+                // will still be successful.
+
+                if ( unused > clusterSize / 2 )
+                    return true;
+            }
+        }
+    }
+
+    return false;
+}
+
+
 QVariant DirTreeModel::sizeColText( FileInfo * item ) const
 {
     if ( item->isDevice() )
 	return QVariant();
 
     if ( item->isDirInfo() )
-	return item->sizePrefix() + formatSize( item->totalSize() );
+	return item->sizePrefix() + formatSize( item->totalAllocatedSize() );
 
     QString text = sizeText( item );
 
+    if ( text.isEmpty() && isSmallFile( item ) )
+	text = smallSizeText( item );
+
     if ( text.isEmpty() )
-        text = formatSize( item->size() );
+	text = formatSize( item->size() );
 
     return text;
 }
@@ -838,21 +935,23 @@ QVariant DirTreeModel::columnIcon( FileInfo * item, int col ) const
     if ( col != NameCol )
 	return QVariant();
 
-    QPixmap icon;
+    QIcon icon;
 
     if	    ( item->isDotEntry() )  icon = _dotEntryIcon;
+    else if ( item->isAttic() )	    icon = _atticIcon;
     else if ( item->isPkgInfo()	 )  icon = _pkgIcon;
     else if ( item->isExcluded() )  icon = _excludedIcon;
     else if ( item->isDir()	 )
     {
 	if	( item->readState() == DirAborted )   icon = _stopIcon;
-	else if ( item->readError()       	  )   icon = _unreadableDirIcon;
+	else if ( item->readError()		  )   icon = _unreadableDirIcon;
 	else if ( item->isMountPoint()		  )   icon = _mountPointIcon;
+	else if ( item->isIgnored()		  )   icon = _atticIcon;
 	else					      icon = _dirIcon;
     }
     else // ! item->isDir()
     {
-	if	( item->readError()     	  )   icon = _unreadableDirIcon;
+	if	( item->readError()		  )   icon = _unreadableDirIcon;
 	else if ( item->isFile()		  )   icon = _fileIcon;
 	else if ( item->isSymLink()		  )   icon = _symlinkIcon;
 	else if ( item->isBlockDevice()		  )   icon = _blockDeviceIcon;
@@ -860,7 +959,14 @@ QVariant DirTreeModel::columnIcon( FileInfo * item, int col ) const
 	else if ( item->isSpecial()		  )   icon = _specialIcon;
     }
 
-    return icon.isNull() ? QVariant() : icon;
+    if ( icon.isNull() )
+	return QVariant();
+
+    bool  useDisabled = item->isIgnored() || item->isAttic();
+    QSize iconSize( icon.actualSize( QSize( 1024, 1024 ) ) );
+
+    return icon.pixmap( iconSize, useDisabled ?
+			QIcon::Disabled : QIcon::Normal );
 }
 
 
