@@ -6,6 +6,9 @@
  *   Author:	Stefan Hundhammer <Stefan.Hundhammer@gmx.de>
  */
 
+
+#include "Qt4Compat.h"
+
 #include "FileDetailsView.h"
 #include "AdaptiveTimer.h"
 #include "DirInfo.h"
@@ -16,10 +19,11 @@
 #include "SystemFileChecker.h"
 #include "Settings.h"
 #include "SettingsHelpers.h"
-#include "Qt4Compat.h"
 #include "Logger.h"
 #include "Exception.h"
 
+#define ALLOCATED_FAT_PERCENT   33
+
 using namespace QDirStat;
 
 
@@ -158,7 +162,7 @@ void FileDetailsView::showFileInfo( FileInfo * file )
     CHECK_PTR( file );
 
     setLabelLimited(_ui->fileNameLabel, file->baseName() );
-    _ui->fileTypeLabel->setText( formatFileSystemObjectType( file->mode() ) );
+    _ui->fileTypeLabel->setText( formatFilesystemObjectType( file->mode() ) );
 
     QString category = mimeCategory( file );
 
@@ -167,11 +171,15 @@ void FileDetailsView::showFileInfo( FileInfo * file )
     _ui->fileMimeCategoryLabel->setText( category );
 
     setFileSizeLabel( _ui->fileSizeLabel, file );
+    setFileAllocatedLabel( _ui->fileAllocatedLabel, file );
+
     _ui->fileUserLabel->setText( file->userName() );
     _ui->fileGroupLabel->setText( file->groupName() );
     _ui->filePermissionsLabel->setText( formatPermissions( file->mode() ) );
-
     _ui->fileMTimeLabel->setText( formatTime( file->mtime() ) );
+
+    if ( ! file->isSparseFile() )
+        suppressIfSameContent( _ui->fileSizeLabel, _ui->fileAllocatedLabel, _ui->fileAllocatedCaption );
 }
 
 
@@ -180,19 +188,50 @@ void FileDetailsView::setFileSizeLabel( FileSizeLabel * label,
 {
     CHECK_PTR( file );
 
-    QString text = DirTreeModel::sizeText( file );
+    QString text;
 
-    if ( text.isEmpty() )  // The normal case: No hard links, not a sparse file
+    if ( file->links() > 1 )
     {
-	label->setValue( file->size() );
-    }
-    else  // The exotic case: Multiple hard links or sparse file or both
-    {
-	label->setText( text );
+        label->setText( tr( "%1 / %2 Links" )
+                        .arg( formatSize( file->rawByteSize() ) )
+                        .arg( file->links() ) );
 
-	if ( file->byteSize() >= 1024 ) // Not useful below 1 kB
-	    label->setContextText( DirTreeModel::sizeText( file, formatByteSize ) );
+	if ( file->rawByteSize() >= 1024 ) // Not useful below 1 kB
+        {
+	    label->setContextText( tr( "%1 / %2 Links" )
+                                   .arg( formatByteSize( file->rawByteSize() ) )
+                                   .arg( file->links() ) );
+        }
     }
+    else
+    {
+        label->setValue( file->rawByteSize() );
+    }
+}
+
+
+void FileDetailsView::setFileAllocatedLabel( FileSizeLabel * label,
+					     FileInfo *	     file )
+{
+    CHECK_PTR( file );
+
+    if ( file->links() > 1 )
+    {
+        label->setText( tr( "%1 / %2 Links" )
+                        .arg( formatSize( file->rawAllocatedSize() ) )
+                        .arg( file->links() ) );
+
+        label->setContextText( tr( "%1 / %2 Links" )
+                               .arg( formatByteSize( file->rawAllocatedSize() ) )
+                               .arg( file->links() ) );
+    }
+    else
+    {
+        label->setValue( file->allocatedSize() );
+    }
+
+    _ui->fileAllocatedLabel->setBold( file->usedPercent() < ALLOCATED_FAT_PERCENT ||
+                                      file->isSparseFile() );
 }
 
 
@@ -236,7 +275,7 @@ void FileDetailsView::updatePkgInfo( const QVariant & pathVariant )
 
 void FileDetailsView::setSystemFileWarningVisibility( bool visible )
 {
-    _ui->fileSystemFileWarning->setVisible( visible );
+    _ui->filesystemFileWarning->setVisible( visible );
     _ui->fileSpacerCaption2->setVisible( visible );
 }
 
@@ -262,6 +301,9 @@ void FileDetailsView::showDetails( DirInfo * dir )
     QString name    = dir->isPseudoDir() ? dir->name() : ( dir->baseName() + "/" );
     QString dirType = dir->isPseudoDir() ? tr( "Pseudo Directory" ) : tr( "Directory" );
 
+    if ( dir->isMountPoint() )
+        dirType = tr( "Mount Point" );
+
     setLabelLimited(_ui->dirNameLabel, name );
     _ui->dirTypeLabel->setText( dirType );
 
@@ -297,31 +339,30 @@ void FileDetailsView::showSubtreeInfo( DirInfo * dir )
 	    break;
     }
 
-#if 0
-    // Also displaying the error message in read seems a bit over the top.
-    setLabelColor( _ui->dirTotalSizeLabel,
-                   dir->readError() ? _dirReadErrColor : _normalTextColor );
-#endif
-
     if ( msg.isEmpty() )
     {
 	// No special msg -> show summary fields
 
 	QString prefix = dir->sizePrefix();
 
-	setLabel( _ui->dirTotalSizeLabel,   dir->totalSize(),	 prefix );
-	setLabel( _ui->dirItemCountLabel,   dir->totalItems(),	 prefix );
-	setLabel( _ui->dirFileCountLabel,   dir->totalFiles(),	 prefix );
-	setLabel( _ui->dirSubDirCountLabel, dir->totalSubDirs(), prefix );
-        _ui->dirLatestMTimeLabel->setText( formatTime( dir->latestMtime() ) );
+	setLabel( _ui->dirTotalSizeLabel,   dir->totalSize(),	       prefix );
+	setLabel( _ui->dirAllocatedLabel,   dir->totalAllocatedSize(), prefix );
+	setLabel( _ui->dirItemCountLabel,   dir->totalItems(),	       prefix );
+	setLabel( _ui->dirFileCountLabel,   dir->totalFiles(),	       prefix );
+	setLabel( _ui->dirSubDirCountLabel, dir->totalSubDirs(),       prefix );
+	_ui->dirLatestMTimeLabel->setText( formatTime( dir->latestMtime() ) );
+
+        suppressIfSameContent( _ui->dirTotalSizeLabel, _ui->dirAllocatedLabel, _ui->dirAllocatedCaption );
+        _ui->dirAllocatedLabel->setBold( dir->totalUsedPercent() < ALLOCATED_FAT_PERCENT );
     }
     else  // Special msg -> show it and clear all summary fields
     {
 	_ui->dirTotalSizeLabel->setText( msg );
+	_ui->dirAllocatedLabel->clear();
 	_ui->dirItemCountLabel->clear();
 	_ui->dirFileCountLabel->clear();
 	_ui->dirSubDirCountLabel->clear();
-        _ui->dirLatestMTimeLabel->clear();
+	_ui->dirLatestMTimeLabel->clear();
     }
 }
 
@@ -346,12 +387,12 @@ void FileDetailsView::showDirNodeInfo( DirInfo * dir )
 	_ui->dirMTimeLabel->setText( formatTime( dir->mtime() ) );
 
 
-        // Show permissions in read if there was a "permission denied" error
-        // while reading this directory
+	// Show permissions in read if there was a "permission denied" error
+	// while reading this directory
 
-        setLabelColor( _ui->dirPermissionsLabel,
-                       dir->readState() == DirPermissionDenied ?
-                       _dirReadErrColor : _normalTextColor );
+	setLabelColor( _ui->dirPermissionsLabel,
+		       dir->readState() == DirPermissionDenied ?
+		       _dirReadErrColor : _normalTextColor );
     }
 }
 
@@ -426,14 +467,18 @@ void FileDetailsView::showDetails( PkgInfo * pkg )
 
 	QString prefix = pkg->sizePrefix();
 
-	setLabel( _ui->pkgTotalSizeLabel,   pkg->totalSize()	);
-	setLabel( _ui->pkgItemCountLabel,   pkg->totalItems()	);
-	setLabel( _ui->pkgFileCountLabel,   pkg->totalFiles()	);
-	setLabel( _ui->pkgSubDirCountLabel, pkg->totalSubDirs() );
+	setLabel( _ui->pkgTotalSizeLabel,   pkg->totalSize()	      );
+	setLabel( _ui->pkgAllocatedLabel,   pkg->totalAllocatedSize() );
+	setLabel( _ui->pkgItemCountLabel,   pkg->totalItems()	      );
+	setLabel( _ui->pkgFileCountLabel,   pkg->totalFiles()	      );
+	setLabel( _ui->pkgSubDirCountLabel, pkg->totalSubDirs()	      );
+
+        suppressIfSameContent( _ui->pkgTotalSizeLabel, _ui->pkgAllocatedLabel, _ui->pkgAllocatedCaption );
     }
     else  // Special msg -> show it and clear all summary fields
     {
 	_ui->pkgTotalSizeLabel->setText( msg );
+	_ui->pkgAllocatedLabel->clear();
 	_ui->pkgItemCountLabel->clear();
 	_ui->pkgFileCountLabel->clear();
 	_ui->pkgSubDirCountLabel->clear();
@@ -465,10 +510,15 @@ void FileDetailsView::showPkgSummary( PkgInfo * pkg )
 
     if ( ! pkg->isBusy() && pkg->readState() == DirFinished )
     {
-	setLabel( _ui->pkgSummaryTotalSizeLabel,   pkg->totalSize()    );
-	setLabel( _ui->pkgSummaryItemCountLabel,   pkg->totalItems()   );
-	setLabel( _ui->pkgSummaryFileCountLabel,   pkg->totalFiles()   );
-	setLabel( _ui->pkgSummarySubDirCountLabel, pkg->totalSubDirs() );
+	setLabel( _ui->pkgSummaryTotalSizeLabel,   pkg->totalSize()	     );
+	setLabel( _ui->pkgSummaryAllocatedLabel,   pkg->totalAllocatedSize() );
+	setLabel( _ui->pkgSummaryItemCountLabel,   pkg->totalItems()	     );
+	setLabel( _ui->pkgSummaryFileCountLabel,   pkg->totalFiles()	     );
+	setLabel( _ui->pkgSummarySubDirCountLabel, pkg->totalSubDirs()	     );
+
+        suppressIfSameContent( _ui->pkgSummaryTotalSizeLabel,
+                               _ui->pkgSummaryAllocatedLabel,
+                               _ui->pkgSummaryAllocatedCaption );
     }
     else
     {
@@ -480,11 +530,12 @@ void FileDetailsView::showPkgSummary( PkgInfo * pkg )
 	}
 	else
 	{
-            if ( pkg->readError() )
+	    if ( pkg->readError() )
 		msg = tr( "[Read Error]" );
 	}
 
 	_ui->pkgSummaryTotalSizeLabel->setText( msg );
+	_ui->pkgSummaryAllocatedLabel->clear();
 	_ui->pkgSummaryItemCountLabel->clear();
 	_ui->pkgSummaryFileCountLabel->clear();
 	_ui->pkgSummarySubDirCountLabel->clear();
@@ -525,11 +576,14 @@ void FileDetailsView::showSelectionSummary( const FileInfoSet & selectedItems )
     _ui->selSubtreeFileCountCaption->setEnabled( subtreeFileCount > 0 );
     _ui->selSubtreeFileCountLabel->setEnabled( subtreeFileCount > 0 );
 
-    setLabel( _ui->selItemCount,	     sel.count()      );
-    setLabel( _ui->selTotalSizeLabel,	     sel.totalSize()  );
-    setLabel( _ui->selFileCountLabel,	     fileCount	      );
-    setLabel( _ui->selDirCountLabel,	     dirCount	      );
-    setLabel( _ui->selSubtreeFileCountLabel, subtreeFileCount );
+    setLabel( _ui->selItemCount,	     sel.count()	      );
+    setLabel( _ui->selTotalSizeLabel,	     sel.totalSize()	      );
+    setLabel( _ui->selAllocatedLabel,	     sel.totalAllocatedSize() );
+    setLabel( _ui->selFileCountLabel,	     fileCount		      );
+    setLabel( _ui->selDirCountLabel,	     dirCount		      );
+    setLabel( _ui->selSubtreeFileCountLabel, subtreeFileCount	      );
+
+    suppressIfSameContent( _ui->selTotalSizeLabel, _ui->selAllocatedLabel, _ui->selAllocatedCaption );
 }
 
 
@@ -574,6 +628,22 @@ QString FileDetailsView::limitText( const QString & longText )
 }
 
 
+void FileDetailsView::suppressIfSameContent( FileSizeLabel * origLabel,
+                                             FileSizeLabel * cloneLabel,
+                                             QLabel        * caption )
+{
+    if ( origLabel->text() == cloneLabel->text() )
+    {
+        cloneLabel->clear();
+        caption->setEnabled( false );
+    }
+    else
+    {
+        caption->setEnabled( true );
+    }
+}
+
+
 QString FileDetailsView::mimeCategory( FileInfo * file )
 {
     MimeCategory * category = MimeCategorizer::instance()->category( file );
