@@ -24,7 +24,7 @@
 #include "Logger.h"
 #include "Exception.h"
 
-// Some file systems (NTFS seems to be among them) may handle block fragments
+// Some filesystems (NTFS seems to be among them) may handle block fragments
 // well. Don't report files as "sparse" files if the block size is only a few
 // bytes less than the byte size - it may be due to intelligent fragment
 // handling.
@@ -34,6 +34,9 @@
 using namespace QDirStat;
 
 
+bool FileInfo::_ignoreHardLinks = false;
+
+
 FileInfo::FileInfo( DirTree    * tree,
 		    DirInfo    * parent,
 		    const char * name )
@@ -41,19 +44,20 @@ FileInfo::FileInfo( DirTree    * tree,
     , _next( 0 )
     , _tree( tree )
 {
-    _isLocalFile  = true;
-    _isSparseFile = false;
-    _isIgnored	  = false;
-    _name	  = name ? name : "";
-    _device	  = 0;
-    _mode	  = 0;
-    _links	  = 0;
-    _uid	  = 0;
-    _gid	  = 0;
-    _size	  = 0;
-    _blocks	  = 0;
-    _mtime	  = 0;
-    _magic	  = FileInfoMagic;
+    _isLocalFile   = true;
+    _isSparseFile  = false;
+    _isIgnored	   = false;
+    _name	   = name ? name : "";
+    _device	   = 0;
+    _mode	   = 0;
+    _links	   = 0;
+    _uid	   = 0;
+    _gid	   = 0;
+    _size	   = 0;
+    _blocks	   = 0;
+    _mtime	   = 0;
+    _allocatedSize = 0;
+    _magic	   = FileInfoMagic;
 }
 
 
@@ -67,17 +71,18 @@ FileInfo::FileInfo( const QString & filenameWithoutPath,
 {
     CHECK_PTR( statInfo );
 
-    _isLocalFile = true;
-    _isIgnored	  = false;
-    _name	 = filenameWithoutPath;
+    _isLocalFile   = true;
+    _isIgnored	   = false;
+    _name	   = filenameWithoutPath;
 
-    _device	 = statInfo->st_dev;
-    _mode	 = statInfo->st_mode;
-    _links	 = statInfo->st_nlink;
-    _uid	 = statInfo->st_uid;
-    _gid	 = statInfo->st_gid;
-    _mtime	 = statInfo->st_mtime;
-    _magic	 = FileInfoMagic;
+    _device	   = statInfo->st_dev;
+    _mode	   = statInfo->st_mode;
+    _links	   = statInfo->st_nlink;
+    _uid	   = statInfo->st_uid;
+    _gid	   = statInfo->st_gid;
+    _mtime	   = statInfo->st_mtime;
+    _magic	   = FileInfoMagic;
+    _allocatedSize = 0;
 
     if ( isSpecial() )
     {
@@ -89,15 +94,34 @@ FileInfo::FileInfo( const QString & filenameWithoutPath,
     {
 	_size		= statInfo->st_size;
 	_blocks		= statInfo->st_blocks;
+
+	if ( _blocks == 0 && _size > 0 )
+	{
+	    if ( ! filesystemCanReportBlocks() )
+	    {
+		_allocatedSize = _size;
+
+		// Do not make any assumptions about fragment handling: The
+		// last block of the file might be partially unused, or the
+		// filesystem might do clever fragment handling, or it's an
+		// exported kernel table like /dev, /proc, /sys. So let's
+		// simply use the size reported by stat() for _allocatedSize.
+	    }
+	}
+	else
+	{
+	    _allocatedSize = _blocks * STD_BLOCK_SIZE;
+	}
+
 	_isSparseFile	= isFile()
-	    && _blocks >= 0				// if file system can report blocks
-	    && allocatedSize() + FRAGMENT_SIZE < _size; // allow for intelligent fragment handling
+	    && _blocks >= 0
+	    && _allocatedSize + FRAGMENT_SIZE < _size; // allow for intelligent fragment handling
 
 	if ( _isSparseFile )
 	{
 	    logDebug() << "Found sparse file: " << this
-		       << "    Byte size: " << formatSize( byteSize() )
-		       << "  Allocated: " << formatSize( allocatedSize() )
+		       << "    Byte size: " << formatSize( _size )
+		       << "  Allocated: " << formatSize( _allocatedSize )
 		       << " (" << (int) _blocks << " blocks)"
 		       << endl;
 	}
@@ -125,25 +149,35 @@ FileInfo::FileInfo( DirTree *	    tree,
     , _next( 0 )
     , _tree( tree )
 {
-    _name	 = filenameWithoutPath;
-    _isLocalFile = true;
-    _isIgnored	 = false;
-    _device	 = 0;
-    _mode	 = mode;
-    _size	 = size;
-    _mtime	 = mtime;
-    _links	 = links;
-    _uid	 = 0;
-    _gid	 = 0;
-    _magic	 = FileInfoMagic;
+    _name	   = filenameWithoutPath;
+    _isLocalFile   = true;
+    _isIgnored	   = false;
+    _device	   = 0;
+    _mode	   = mode;
+    _size	   = size;
+    _mtime	   = mtime;
+    _allocatedSize = 0;
+    _links	   = links;
+    _uid	   = 0;
+    _gid	   = 0;
+    _magic	   = FileInfoMagic;
 
     if ( blocks < 0 )
     {
 	_isSparseFile	= false;
-	_blocks		= _size / blockSize();
+	_blocks		= _size / STD_BLOCK_SIZE;
 
-	if ( ( _size % blockSize() ) > 0 )
+	if ( ( _size % STD_BLOCK_SIZE ) > 0 )
 	    _blocks++;
+
+	// Don't make any assumptions about the file's tail. We might use
+	//
+	//   _allocatedSize = _blocks * STD_BLOCK_SIZE;
+	//
+	// but that might be wrong if the filesystem has intelligent fragment
+	// handling. Simply use the byte size instead.
+
+	_allocatedSize = _size;
     }
     else
     {
@@ -178,23 +212,41 @@ bool FileInfo::checkMagicNumber() const
 }
 
 
-FileSize FileInfo::allocatedSize() const
-{
-    return blocks() * blockSize();
-}
-
-
 FileSize FileInfo::size() const
 {
-    FileSize sz = _isSparseFile ? allocatedSize() : _size;
+    FileSize sz = _isSparseFile ? _allocatedSize : _size;
 
-    if ( _links > 1 && ! isDir() )
+    if ( _links > 1 && ! _ignoreHardLinks && isFile() )
 	sz /= _links;
 
     return sz;
 }
 
 
+FileSize FileInfo::allocatedSize() const
+{
+    FileSize sz = _allocatedSize;
+
+    if ( _links > 1 && ! _ignoreHardLinks && isFile() )
+	sz /= _links;
+
+    return sz;
+}
+
+
+int FileInfo::usedPercent() const
+{
+    int percent = 100;
+
+    if ( _allocatedSize > 0 && _size > 0 )
+    {
+        percent = qRound( ( 100.0 * size() ) / allocatedSize() );
+    }
+
+    return percent;
+}
+
+
 QString FileInfo::url() const
 {
     if ( _parent )
@@ -394,6 +446,20 @@ float FileInfo::subtreePercent()
 }
 
 
+float FileInfo::subtreeAllocatedPercent()
+{
+    if ( ! parent()			     ||	// only if there is a parent as calculation base
+	 parent()->pendingReadJobs() > 0     ||	// not before subtree is finished reading
+	 parent()->totalAllocatedSize() == 0 ||	// avoid division by zero
+	 isExcluded() )				// not if this is an excluded object (dir)
+    {
+	return -1.0;
+    }
+
+    return ( 100.0 * totalAllocatedSize() ) / (float) parent()->totalAllocatedSize();
+}
+
+
 QString FileInfo::dotEntryName()
 {
     return QObject::tr( "<Files>" );
@@ -474,10 +540,25 @@ QString FileInfo::baseName() const
 }
 
 
+void FileInfo::setIgnoreHardLinks( bool ignore )
+{
+    if ( ignore )
+	logInfo() << "Ignoring hard links" << endl;
+
+    _ignoreHardLinks = ignore;
+}
+
+
 //---------------------------------------------------------------------------
 
 
 QString QDirStat::formatSize( FileSize lSize )
+{
+    return formatSize( lSize, 1 );
+}
+
+
+QString QDirStat::formatSize( FileSize lSize, int precision )
 {
     QString sizeString;
     int	    unitIndex = 0;
@@ -512,7 +593,7 @@ QString QDirStat::formatSize( FileSize lSize )
 	    ++unitIndex;
 	}
 
-	sizeString.setNum( size, 'f', 1 );
+	sizeString.setNum( size, 'f', precision );
 	sizeString += " " + units.at( unitIndex );
     }
 
@@ -593,6 +674,31 @@ PkgInfo * FileInfo::pkgInfoParent() const
 }
 
 
+bool FileInfo::filesystemCanReportBlocks() const
+{
+    const FileInfo * dir = this;
+
+    // Find the nearest real directory from here;
+    // do not use a DotEntry or an Attic because they always have 0 blocks.
+
+    while ( ! dir->isDirInfo() || dir->isPseudoDir() )
+    {
+	dir = dir ->parent();
+
+	if ( ! dir )
+	    return false;
+    }
+
+    // logDebug() << "Checking block size of " << dir << ": " << (int) dir->blocks() << endl;
+
+    // A real directory never has a size == 0, so we can skip this check.
+
+    return dir->blocks() > 0;
+}
+
+
+
+
 QString QDirStat::formatTime( time_t rawTime )
 {
     if ( rawTime == (time_t) 0 )
@@ -672,7 +778,7 @@ QString QDirStat::symbolicMode( mode_t mode, bool omitTypeForRegularFiles )
 }
 
 
-QString QDirStat::formatFileSystemObjectType( mode_t mode )
+QString QDirStat::formatFilesystemObjectType( mode_t mode )
 {
     if	    ( S_ISDIR ( mode ) ) return QObject::tr( "Directory"	);
     else if ( S_ISCHR ( mode ) ) return QObject::tr( "Character Device" );
