@@ -20,6 +20,14 @@
 
 #include "Logger.h"
 
+// The size of a standard disk block.
+//
+// Notice that this is different from st_blksize in the struct that the stat()
+// syscall returns, yet it is the reference unit for st_blocks in that same
+// struct.
+
+#define STD_BLOCK_SIZE	512L
+
 
 namespace QDirStat
 {
@@ -49,7 +57,7 @@ namespace QDirStat
 	DirOnRequestOnly,	// Will be read upon explicit request only (mount points)
 	DirCached,		// Content was read from a cache
 	DirAborted,		// Reading aborted upon user request
-        DirPermissionDenied,    // Insufficient permissions for reading
+	DirPermissionDenied,	// Insufficient permissions for reading
 	DirError		// Error while reading
     };
 
@@ -61,7 +69,7 @@ namespace QDirStat
      * typically obtained by stat() / lstat() or similar calls.
      *
      * This class is tuned for size rather than speed: A typical Linux system
-     * easily has 150,000+ file system objects, and at least one entry of this
+     * easily has 150,000+ filesystem objects, and at least one entry of this
      * sort is required for each of them.
      *
      * This class provides stubs for children management, yet those stubs all
@@ -258,33 +266,6 @@ namespace QDirStat
 	 **/
 	QString octalPermissions() const;
 
-	/**
-	 * The file size in bytes. This does not take unused space in the last
-	 * disk block (cluster) into account, yet it is the only size all kinds
-	 * of info functions can obtain. This is also what most file system
-	 * utilities (like "ls -l") display.
-	 **/
-	FileSize byteSize() const { return _size;   }
-
-	/**
-	 * The number of bytes actually allocated on the file system. Usually
-	 * this will be more than byteSize() since the last few bytes of a file
-	 * usually consume an additional cluster on the file system.
-	 *
-	 * In the case of sparse files, however, this might as well be
-	 * considerably less than byteSize() - this means that this file has
-	 * "holes", i.e. large portions filled with zeros. This is typical for
-	 * large core dumps for example. The only way to create such a file is
-	 * to lseek() far ahead of the previous file size and then writing
-	 * data. Most file system utilities will however disregard the fact
-	 * that files are sparse files and simply allocate the holes as well,
-	 * thus greatly increasing the disk space consumption of such a
-	 * file. Only some few file system utilities like "cp", "rsync", "tar"
-	 * have options to handle this more graciously - but usually only when
-	 * specifically requested. See the respective man pages.
-	 **/
-	FileSize allocatedSize() const;
-
 	/**
 	 * The file size, taking into account multiple links for plain files or
 	 * the true allocated size for sparse files. For plain files with
@@ -293,17 +274,51 @@ namespace QDirStat
 	 **/
 	FileSize size() const;
 
+	/**
+	 * The file size in bytes without taking multiple hard links into
+	 * account.
+	 **/
+	FileSize rawByteSize() const { return _size; }
+
+	/**
+	 * The number of bytes actually allocated on the filesystem, taking
+	 * multiple hard links (for plain files) into account.
+	 *
+	 * Usually this will be more than size() since the last few bytes of a
+	 * file usually consume an additional cluster on the filesystem.
+	 *
+	 * In the case of sparse files, however, this might as well be
+	 * considerably less than size() - this means that this file has
+	 * "holes", i.e. large portions filled with zeros. This is typical for
+	 * large core dumps for example. The only way to create such a file is
+	 * to lseek() far ahead of the previous file size and then writing
+	 * data. Most filesystem utilities will however disregard the fact that
+	 * files are sparse files and simply allocate the holes as well, thus
+	 * greatly increasing the disk space consumption of such a file. Only
+	 * some few filesystem utilities like "cp", "rsync", "tar" have options
+	 * to handle this more graciously - but usually only when specifically
+	 * requested. See the respective man pages.
+	 **/
+	FileSize allocatedSize() const;
+
+        /**
+         * The ratio of size() / allocatedSize() in percent.
+         **/
+        int usedPercent() const;
+
+	/**
+	 * The allocated size without taking multiple hard links into account.
+	 *
+	 * If the filesystem can properly report the number of disk blocks
+	 * used, this is the same as blocks() * 512.
+	 **/
+	FileSize rawAllocatedSize() const { return _allocatedSize; }
+
 	/**
 	 * The file size in 512 byte blocks.
 	 **/
 	FileSize blocks() const { return _blocks; }
 
-	/**
-	 * The size of one single block that blocks() returns.
-	 * Notice: This is _not_ the blocksize that lstat() returns!
-	 **/
-	FileSize blockSize() const { return 512L;    }
-
 	/**
 	 * The modification time of the file (not the inode).
 	 **/
@@ -315,6 +330,12 @@ namespace QDirStat
 	 **/
 	virtual FileSize totalSize() { return size(); }
 
+	/**
+	 * Returns the total allocated size in bytes of this subtree.
+	 * Derived classes that have children should overwrite this.
+	 **/
+	virtual FileSize totalAllocatedSize() { return allocatedSize(); }
+
 	/**
 	 * Returns the total size in blocks of this subtree.
 	 * Derived classes that have children should overwrite this.
@@ -371,16 +392,16 @@ namespace QDirStat
 	 **/
 	virtual int directChildrenCount() { return 0; }
 
-        /**
-         * Returns the number of subdirectories below this item that could not
-         * be read (typically due to insufficient permissions).
-         *
-         * Notice that this does NOT include this item if it is a directory
-         * that could not be read.
+	/**
+	 * Returns the number of subdirectories below this item that could not
+	 * be read (typically due to insufficient permissions).
+	 *
+	 * Notice that this does NOT include this item if it is a directory
+	 * that could not be read.
 	 *
 	 * Derived classes that have children should overwrite this.
-         **/
-        virtual int errSubDirCount() { return 0; }
+	 **/
+	virtual int errSubDirCount() { return 0; }
 
 	/**
 	 * Returns the latest modification time of this subtree.
@@ -391,10 +412,10 @@ namespace QDirStat
 	/**
 	 * Returns the oldest modification time of any file in this subtree or
 	 * 0 if there is no file.
-         *
-         * Derived classes that have children should overwrite this.
+	 *
+	 * Derived classes that have children should overwrite this.
 	 **/
-        virtual time_t oldestFileMtime() { return isFile() ? _mtime : 0; }
+	virtual time_t oldestFileMtime() { return isFile() ? _mtime : 0; }
 
 	/**
 	 * Return the percentage of this subtree in regard to its parent
@@ -406,6 +427,17 @@ namespace QDirStat
 	 **/
 	virtual float subtreePercent();
 
+	/**
+	 * Return the percentage of this subtree's allocated size in regard to
+	 * its parent's allocated size.  (0.0..100.0). Return a negative value
+	 * if for any reason this cannot be calculated or it would not make any
+	 * sense.
+	 *
+	 * Derived classes are free to overwrite this, but this default
+	 * implementation should work well enough.
+	 **/
+	virtual float subtreeAllocatedPercent();
+
 	/**
 	 * Returns 'true' if this had been excluded while reading.
 	 * Derived classes may want to overwrite this.
@@ -458,7 +490,16 @@ namespace QDirStat
 	 * number reaches zero, the entire subtree is done.
 	 * Derived classes that have children should overwrite this.
 	 **/
-	virtual int pendingReadJobs() { return 0;  }
+	virtual int pendingReadJobs() { return 0; }
+
+	/**
+	 * Return 'true' if the filesystem can report block sizes.
+	 *
+	 * This is determined heuristically from the nearest DirInfo parent: If
+	 * it has blocks() > 0 and size() > 0, we can safely assume that the
+	 * filesystem does report the number of blocks.
+	 **/
+	bool filesystemCanReportBlocks() const;
 
 
 	//
@@ -652,25 +693,25 @@ namespace QDirStat
 	 **/
 	virtual DirReadState readState() const { return DirFinished; }
 
-        /**
-         * Check if readState() is anything that indicates an error reading the
-         * directory, i.e. DirError or DirPermissionDenied.
-         *
-         * This default implementation always returns 'false'.
+	/**
+	 * Check if readState() is anything that indicates an error reading the
+	 * directory, i.e. DirError or DirPermissionDenied.
+	 *
+	 * This default implementation always returns 'false'.
 	 * Derived classes should overwrite this.
-         **/
-        virtual bool readError() const { return false; }
+	 **/
+	virtual bool readError() const { return false; }
 
-        /**
-         * Return a prefix for the total size (and similar accumulated fields)
-         * of this item: ">" if there might be more, i.e. if a subdirectory
-         * could not be read or if reading was aborted, an empty string
-         * otherwise.
-         *
-         * This default implementation returns an empty string. Derived classes
-         * that can handle child items should reimplement this.
-         **/
-        virtual QString sizePrefix() const { return ""; }
+	/**
+	 * Return a prefix for the total size (and similar accumulated fields)
+	 * of this item: ">" if there might be more, i.e. if a subdirectory
+	 * could not be read or if reading was aborted, an empty string
+	 * otherwise.
+	 *
+	 * This default implementation returns an empty string. Derived classes
+	 * that can handle child items should reimplement this.
+	 **/
+	virtual QString sizePrefix() const { return ""; }
 
 	/**
 	 * Returns true if this is a DirInfo object.
@@ -809,6 +850,34 @@ namespace QDirStat
 						 S_ISFIFO( _mode ) ||
 						 S_ISSOCK( _mode )   ) ? true : false; }
 
+	/**
+	 * Set the policy how hard links are handled: By default, for files
+	 * with multiple hard links, the size is distributed among each
+	 * individual hard link for that file. So a file with a size of 4 kB
+	 * and 4 hard links reports 1 kB to its parent directory.
+	 *
+	 * When this flag is set to 'true', it will report the full 4 kB each
+	 * time, so all 4 hard links together will now add up to 16 kB. While
+	 * this is probably a very bad idea if those links are all in the same
+	 * directory (or subtree), it might even be useful if there are several
+	 * separate subtrees that all share hard links between each other, but
+	 * not within the same subtree. Some backup systems use this strategy
+	 * to save disk space.
+	 *
+	 * Use this with caution.
+	 *
+	 * This flag will be read from the config file from the outside
+	 * (DirTree) and set from there using this function.
+	 **/
+	static void setIgnoreHardLinks( bool ignore );
+
+	/**
+	 * Return the current hard links accounting policy.
+	 * See setIgnoreHardLinks() for details.
+	 **/
+	static bool ignoreHardLinks() { return _ignoreHardLinks; }
+
+
     protected:
 
 	// Data members.
@@ -829,11 +898,14 @@ namespace QDirStat
 	FileSize	_size;			// size in bytes
 	FileSize	_blocks;		// 512 bytes blocks
 	time_t		_mtime;			// modification time
+	FileSize	_allocatedSize;		// allocated size in bytes
 
 	DirInfo	 *	_parent;		// pointer to the parent entry
 	FileInfo *	_next;			// pointer to the next entry
 	DirTree	 *	_tree;			// pointer to the parent tree
 
+	static bool	_ignoreHardLinks;	// don't distribute size for multiple hard links
+
     };	// class FileInfo
 
 
@@ -849,13 +921,18 @@ namespace QDirStat
 
     /**
      * Format a file / subtree size human readable, i.e. in "GB" / "MB"
-     * etc. rather than huge numbers of digits.
+     * etc. rather than huge numbers of digits. 'precision' is the number of
+     * digits after the decimal point.
      *
      * Note: For logDebug() etc., operator<< is overwritten to do exactly that:
      *
      *	   logDebug() << "Size: " << x->totalSize() << endl;
      **/
-    QString formatSize ( FileSize size );
+    QString formatSize( FileSize size );
+
+    // Can't use a default argument when using this as a function pointer,
+    // so we really need the above overloaded version.
+    QString formatSize( FileSize size, int precision );
 
     /**
      * Format a file / subtree size as bytes, but still human readable with a
@@ -915,7 +992,7 @@ namespace QDirStat
      * Format the filesystem object type from a mode, e.g. "Directory",
      * "Symbolic Link", "Block Device", "File".
      **/
-    QString formatFileSystemObjectType( mode_t mode );
+    QString formatFilesystemObjectType( mode_t mode );
 
 
     /**
