@@ -73,7 +73,7 @@ void GeneralConfigPage::readSettings()
 
     settings.beginGroup( "DirectoryTree" );
 
-    _ui->crossFileSystemsCheckBox->setChecked( settings.value( "CrossFileSystems"    , false ).toBool() );
+    _ui->crossFilesystemsCheckBox->setChecked( settings.value( "CrossFilesystems"    , false ).toBool() );
     _ui->treeUpdateIntervalSpinBox->setValue ( settings.value( "UpdateTimerMillisec" ,   333 ).toInt()  );
     QString treeIconDir = settings.value( "TreeIconDir", ":/icons/tree-medium/" ).toString();
 
@@ -100,7 +100,7 @@ void GeneralConfigPage::writeSettings()
 
     settings.beginGroup( "DirectoryTree" );
 
-    settings.setValue( "CrossFileSystems"    , _ui->crossFileSystemsCheckBox->isChecked() );
+    settings.setValue( "CrossFilesystems"    , _ui->crossFilesystemsCheckBox->isChecked() );
     settings.setValue( "UpdateTimerMillisec" , _ui->treeUpdateIntervalSpinBox->value()    );
 
     switch ( _ui->treeIconThemeComboBox->currentIndex() )
