@@ -36,6 +36,7 @@
 #include "Logger.h"
 #include "MimeCategorizer.h"
 #include "MimeCategoryConfigPage.h"
+#include "OpenDirDialog.h"
 #include "OpenPkgDialog.h"
 #include "OutputWindow.h"
 #include "PanelMessage.h"
@@ -60,6 +61,8 @@
 #  define HAVE_SIGNAL_MAPPER	  0
 #endif
 
+#define USE_CUSTOM_OPEN_DIR_DIALOG 1
+
 using namespace QDirStat;
 
 using QDirStat::DataColumns;
@@ -90,7 +93,9 @@ MainWindow::MainWindow():
     createLayouts();
     readSettings();
     _updateTimer.setInterval( UPDATE_MILLISEC );
+    _treeExpandTimer.setSingleShot( true );
     _dUrl = _ui->actionDonate->iconText();
+    _futureSelection.setUseRootFallback( false );
 
     _dirTreeModel = new DirTreeModel( this );
     CHECK_NEW( _dirTreeModel );
@@ -221,6 +226,9 @@ void MainWindow::connectSignals()
     connect( &_updateTimer,		SIGNAL( timeout()	  ),
 	     this,			SLOT  ( showElapsedTime() ) );
 
+    connect( &_treeExpandTimer,		  SIGNAL( timeout()	  ),
+             _ui->actionExpandTreeLevel1, SLOT( trigger()          ) );
+
     if ( _useTreemapHover )
     {
 	connect( _ui->treemapView,	SIGNAL( hoverEnter ( FileInfo * ) ),
@@ -319,6 +327,8 @@ void MainWindow::connectActions()
 
     _ui->actionFileTypeStats->setShortcutContext( Qt::ApplicationShortcut );
 
+    CONNECT_ACTION( _ui->actionShowFilesystems,	   this, showFilesystems() );
+
 
 
     // "Edit" menu
@@ -621,20 +631,28 @@ void MainWindow::busyDisplay()
     if ( ! PkgFilter::isPkgUrl( _dirTreeModel->tree()->url() ) &&
 	 ! _selectionModel->currentBranch() )
     {
-	// Wait until the toplevel entry has some children, then expand to level 1
-	QTimer::singleShot( 200, _ui->actionExpandTreeLevel1, SLOT( trigger() ) );
+        _treeExpandTimer.start( 200 );
+        // This will trigger actionExpandTreeLevel1. Hopefully after those 200
+        // millisec there will be some items in the tree to expand.
     }
 }
 
 
 void MainWindow::idleDisplay()
 {
+    logInfo() << endl;
+
     updateActions();
     _updateTimer.stop();
     int sortCol = QDirStat::DataColumns::toViewCol( QDirStat::PercentNumCol );
     _ui->dirTreeView->sortByColumn( sortCol, Qt::DescendingOrder );
 
-    if ( ! _selectionModel->currentBranch() )
+    if ( ! _futureSelection.isEmpty() )
+    {
+        _treeExpandTimer.stop();
+        applyFutureSelection();
+    }
+    else if ( ! _selectionModel->currentBranch() )
     {
 	logDebug() << "No current branch - expanding tree to level 1" << endl;
 	expandTreeToLevel( 1 );
@@ -726,12 +744,22 @@ void MainWindow::openDir( const QString & url )
 
 void MainWindow::askOpenDir()
 {
-    QString url = QFileDialog::getExistingDirectory( this, // parent
-						     tr("Select directory to scan") );
-    if ( ! url.isEmpty() )
+    QString path;
+    DirTree * tree = _dirTreeModel->tree();
+    bool crossFilesystems = tree->crossFilesystems();
+
+#if USE_CUSTOM_OPEN_DIR_DIALOG
+    path = QDirStat::OpenDirDialog::askOpenDir( &crossFilesystems, this );
+#else
+    path = QFileDialog::getExistingDirectory( this, // parent
+                                              tr("Select directory to scan") );
+#endif
+
+    if ( ! path.isEmpty() )
     {
-	_dirTreeModel->tree()->reset();
-	openUrl( url );
+	tree->reset();
+	tree->setCrossFilesystems( crossFilesystems );
+	openUrl( path );
     }
 }
 
@@ -879,11 +907,30 @@ void MainWindow::refreshAll()
 void MainWindow::refreshSelected()
 {
     busyDisplay();
+    _futureSelection.set( _selectionModel->selectedItems().first() );
+    // logDebug() << "Setting future selection: " << _futureSelection.subtree() << endl;
     _dirTreeModel->refreshSelected();
     updateActions();
 }
 
 
+void MainWindow::applyFutureSelection()
+{
+    FileInfo * sel = _futureSelection.subtree();
+    // logDebug() << "Using future selection: " << sel << endl;
+
+    if ( sel )
+    {
+        _treeExpandTimer.stop();
+        _futureSelection.clear();
+        _selectionModel->setCurrentBranch( sel );
+
+        if ( sel->isMountPoint() )
+            _ui->dirTreeView->setExpanded( sel, true );
+    }
+}
+
+
 void MainWindow::stopReading()
 {
     if ( _dirTreeModel->tree()->isBusy() )
@@ -1202,6 +1249,24 @@ void MainWindow::showFileSizeStats()
 }
 
 
+void MainWindow::showFilesystems()
+{
+    if ( ! _filesystemsWindow )
+    {
+	// This deletes itself when the user closes it. The associated QPointer
+	// keeps track of that and sets the pointer to 0 when it happens.
+
+	_filesystemsWindow = new FilesystemsWindow( this );
+
+        connect( _filesystemsWindow, SIGNAL( readFilesystem( QString ) ),
+                 this,               SLOT  ( openUrl       ( QString ) ) );
+    }
+
+    _filesystemsWindow->populate();
+    _filesystemsWindow->show();
+}
+
+
 void MainWindow::initLayoutActions()
 {
     // Qt Designer does not support QActionGroups; it was there for Qt 3, but
