@@ -18,9 +18,11 @@
 
 #include "ui_main-window.h"
 #include "FileTypeStatsWindow.h"
+#include "FilesystemsWindow.h"
 #include "PanelMessage.h"
 #include "UnreadableDirsWindow.h"
 #include "PkgFilter.h"
+#include "Subtree.h"
 
 
 class QCloseEvent;
@@ -43,6 +45,7 @@ using QDirStat::FileInfo;
 using QDirStat::FileTypeStatsWindow;
 using QDirStat::PanelMessage;
 using QDirStat::UnreadableDirsWindow;
+using QDirStat::FilesystemsWindow;
 
 
 class MainWindow: public QMainWindow
@@ -314,6 +317,11 @@ protected slots:
      **/
     void showFileSizeStats();
 
+    /**
+     * Show detailed information about mounted filesystems in a separate window.
+     **/
+    void showFilesystems();
+
     /**
      * Change the main window layout. If no name is passed, the function tries
      * to check if the sender is a QAction and use its data().
@@ -347,6 +355,12 @@ protected slots:
      **/
     void toggleVerboseSelection();
 
+    /**
+     * Apply the future selection: Select the URL that was stored in
+     * _futureSelection, open that branch and clear _futureSelection.
+     **/
+    void applyFutureSelection();
+
     /**
      * Open a popup dialog with a message that this feature is not implemented.
      **/
@@ -441,6 +455,7 @@ private:
     QDirStat::ConfigDialog	*  _configDialog;
     QActionGroup		*  _layoutActionGroup;
     QPointer<FileTypeStatsWindow>  _fileTypeStatsWindow;
+    QPointer<FilesystemsWindow>    _filesystemsWindow;
     QPointer<PanelMessage>	   _dirPermissionsWarning;
     QPointer<UnreadableDirsWindow> _unreadableDirsWindow;
     QString			   _dUrl;
@@ -456,6 +471,8 @@ private:
     QMap<QString, TreeLayout *>	   _layouts;
     TreeLayout *		   _currentLayout;
     QTimer			   _updateTimer;
+    QTimer                         _treeExpandTimer;
+    QDirStat::Subtree              _futureSelection;
 
 }; // class MainWindow
 
