@@ -25,11 +25,7 @@ PercentBarDelegate::PercentBarDelegate( QTreeView * treeView ):
     QStyledItemDelegate( 0 ),
     _treeView( treeView )
 {
-    _percentBarCol = DataColumns::toViewCol( PercentBarCol );
     readSettings();
-
-    connect( DataColumns::instance(), SIGNAL( columnsChanged() ),
-	     this,		      SLOT  ( columnsChanged() ) );
 }
 
 
@@ -80,23 +76,17 @@ void PercentBarDelegate::writeSettings()
 
     writeColorListEntry( settings, "Colors"    , _fillColors	);
     writeColorEntry    ( settings, "Background", _barBackground );
-    settings.setValue( "PercentBarColumnWidth", _sizeHintWidth	);
+    settings.setValue( "PercentBarColumnWidth",  _sizeHintWidth	);
 
     settings.endGroup();
 }
 
 
-void PercentBarDelegate::columnsChanged()
-{
-    _percentBarCol = DataColumns::toViewCol( PercentBarCol );
-}
-
-
 void PercentBarDelegate::paint( QPainter		   * painter,
 				const QStyleOptionViewItem & option,
 				const QModelIndex	   & index ) const
 {
-    if ( ! index.isValid() || index.column() != _percentBarCol )
+    if ( ! index.isValid() || index.column() != PercentBarCol )
 	return QStyledItemDelegate::paint( painter, option, index );
 
     QVariant data = index.data( RawDataRole );
@@ -154,7 +144,7 @@ QSize PercentBarDelegate::sizeHint( const QStyleOptionViewItem & option,
 {
     QSize size = QStyledItemDelegate::sizeHint( option, index );
 
-    if ( ! index.isValid() || index.column() != _percentBarCol )
+    if ( ! index.isValid() || index.column() != PercentBarCol )
 	return size;
 
     size.setWidth( _sizeHintWidth );
@@ -202,7 +192,7 @@ namespace QDirStat
 			       barBackground );
 	    /*
 	     * Notice: The Xlib XDrawRectangle() function always fills one
-	     * pixel less than specified. Altough this is very likely just a
+	     * pixel less than specified. Although this is very likely just a
 	     * plain old bug, it is documented that way. Obviously, Qt just
 	     * maps the fillRect() call directly to XDrawRectangle() so they
 	     * inherited that bug (although the Qt doc stays silent about
