@@ -34,7 +34,6 @@ TreemapView::TreemapView( QWidget * parent ):
     _selectionModel(0),
     _selectionModelProxy(0),
     _cleanupCollection(0),
-    _mimeCategorizer(0),
     _rebuilder(0),
     _rootTile(0),
     _currentItem(0),
@@ -61,7 +60,7 @@ TreemapView::TreemapView( QWidget * parent ):
     CHECK_NEW( _rebuilder );
 
     connect( _rebuilder, SIGNAL( rebuild() ),
-             this,       SLOT  ( rebuildTreemapDelayed() ) );
+	     this,	 SLOT  ( rebuildTreemapDelayed() ) );
 }
 
 
@@ -138,7 +137,7 @@ void TreemapView::setSelectionModel( SelectionModel * selectionModel )
 	     _selectionModel, SLOT  ( setCurrentItem	( FileInfo * ) ) );
 
     connect( this,	      SIGNAL( currentItemChanged( FileInfo * ) ),
-	     _selectionModel, SLOT  ( setCurrentBranch  ( FileInfo * ) ) );
+	     _selectionModel, SLOT  ( setCurrentBranch	( FileInfo * ) ) );
 
 
     // Use the proxy for all receiving signals!
@@ -162,7 +161,7 @@ void TreemapView::readSettings()
     _doCushionShading	= settings.value( "CushionShading"   , true  ).toBool();
     _ensureContrast	= settings.value( "EnsureContrast"   , true  ).toBool();
     _forceCushionGrid	= settings.value( "ForceCushionGrid" , false ).toBool();
-    _useDirGradient     = settings.value( "UseDirGradient"   , true  ).toBool();
+    _useDirGradient	= settings.value( "UseDirGradient"   , true  ).toBool();
     _minTileSize	= settings.value( "MinTileSize"	     , DefaultMinTileSize ).toInt();
 
     _currentItemColor	= readColorEntry( settings, "CurrentItemColor"	, Qt::red		     );
@@ -171,8 +170,8 @@ void TreemapView::readSettings()
     _outlineColor	= readColorEntry( settings, "OutlineColor"	, Qt::black		     );
     _fileFillColor	= readColorEntry( settings, "FileFillColor"	, QColor( 0xde, 0x8d, 0x53 ) );
     _dirFillColor	= readColorEntry( settings, "DirFillColor"	, QColor( 0x10, 0x7d, 0xb4 ) );
-    _dirGradientStart   = readColorEntry( settings, "DirGradientStart"	, QColor( 0x60, 0x60, 0x70 ) );
-    _dirGradientEnd     = readColorEntry( settings, "DirGradientEnd"	, QColor( 0x70, 0x70, 0x80 ) );
+    _dirGradientStart	= readColorEntry( settings, "DirGradientStart"	, QColor( 0x60, 0x60, 0x70 ) );
+    _dirGradientEnd	= readColorEntry( settings, "DirGradientEnd"	, QColor( 0x70, 0x70, 0x80 ) );
 
     settings.endGroup();
 }
@@ -180,7 +179,7 @@ void TreemapView::readSettings()
 
 void TreemapView::writeSettings()
 {
-    logDebug() << endl;
+    // logDebug() << endl;
 
     Settings settings;
     settings.beginGroup( "Treemaps" );
@@ -191,7 +190,7 @@ void TreemapView::writeSettings()
     settings.setValue( "CushionShading"	   , _doCushionShading	 );
     settings.setValue( "EnsureContrast"	   , _ensureContrast	 );
     settings.setValue( "ForceCushionGrid"  , _forceCushionGrid	 );
-    settings.setValue( "UseDirGradient"    , _useDirGradient     );
+    settings.setValue( "UseDirGradient"	   , _useDirGradient	 );
     settings.setValue( "MinTileSize"	   , _minTileSize	 );
 
     writeColorEntry( settings, "CurrentItemColor"  , _currentItemColor	 );
@@ -200,8 +199,8 @@ void TreemapView::writeSettings()
     writeColorEntry( settings, "OutlineColor"	   , _outlineColor	 );
     writeColorEntry( settings, "FileFillColor"	   , _fileFillColor	 );
     writeColorEntry( settings, "DirFillColor"	   , _dirFillColor	 );
-    writeColorEntry( settings, "DirGradientStart"  , _dirGradientStart   );
-    writeColorEntry( settings, "DirGradientEnd"    , _dirGradientEnd     );
+    writeColorEntry( settings, "DirGradientStart"  , _dirGradientStart	 );
+    writeColorEntry( settings, "DirGradientEnd"	   , _dirGradientEnd	 );
 
     settings.endGroup();
 }
@@ -215,7 +214,7 @@ void TreemapView::zoomIn()
     TreemapTile * newRootTile = _currentItem;
 
     while ( newRootTile &&
-            newRootTile->parentTile() != _rootTile &&
+	    newRootTile->parentTile() != _rootTile &&
 	    newRootTile->parentTile() ) // This should never happen, but who knows?
     {
 	newRootTile = newRootTile->parentTile();
@@ -225,7 +224,7 @@ void TreemapView::zoomIn()
     {
 	FileInfo * newRoot = newRootTile->orig();
 
-	if ( newRoot->isDir() || newRoot->isDotEntry() )
+	if ( newRoot->isDirInfo() )
 	    rebuildTreemap( newRoot );
     }
 }
@@ -272,7 +271,7 @@ bool TreemapView::canZoomIn() const
     {
 	FileInfo * newRoot = newRootTile->orig();
 
-	if ( newRoot->isDir() || newRoot->isDotEntry() )
+	if ( newRoot->isDirInfo() )
 	    return true;
     }
 
@@ -297,7 +296,7 @@ void TreemapView::rebuildTreemap()
     {
 	// logDebug() << "Restoring old treemap with root " << _savedRootUrl << endl;
 
-	root = _tree->locate( _savedRootUrl, true );	// node, findDotEntries
+	root = _tree->locate( _savedRootUrl, true );	// node, findPseudoDirs
     }
 
     if ( ! root )
@@ -632,15 +631,6 @@ QSize TreemapView::visibleSize()
 }
 
 
-void TreemapView::setMimeCategorizer( MimeCategorizer * newCategorizer )
-{
-    if ( _mimeCategorizer )
-	delete _mimeCategorizer;
-
-    _mimeCategorizer = newCategorizer;
-}
-
-
 void TreemapView::setFixedColor( const QColor & color )
 {
     _fixedColor	   = color;
@@ -657,13 +647,7 @@ QColor TreemapView::tileColor( FileInfo * file )
     {
 	if ( file->isFile() )
 	{
-	    if ( ! _mimeCategorizer )
-	    {
-		_mimeCategorizer = new MimeCategorizer( this );
-		CHECK_NEW( _mimeCategorizer );
-	    }
-
-	    MimeCategory * category = _mimeCategorizer->category( file );
+	    MimeCategory * category = MimeCategorizer::instance()->category( file );
 
 	    if ( category )
 		return category->color();
